<?php

namespace Drupal\formio;

use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\formio\Plugin\ActionPluginInterface;
use Drupal\Core\Plugin\Factory\ContainerFactory;
use Drupal\Core\Plugin\Discovery\YamlDiscovery;
use Drupal\plug\Util\Module;

/**
 * FormioAction plugin manager.
 */
class ActionPluginManager extends DefaultPluginManager implements ActionPluginInterface {

  /**
   * {@inheritdoc}
   */
  protected $defaults = array(
    'id' => '',
    'label' => '',
    'name' => '',
    'class' => 'Drupal\formio\Plugin\action\FormioAction',
  );

  protected $title;

  protected $action;

  protected $hasMenu;

  protected $callback;

  protected $class;

  /**
   * {@inheritdoc}
   */
  public function __construct(\Traversable $namespaces, \DrupalCacheInterface $cache_backend) {
    parent::__construct(FALSE, $namespaces);
    $this->discovery = new YamlDiscovery('actions', Module::getDirectories());
    $this->factory = new ContainerFactory($this);
    // Allows invoking an alter hook.
    $this->alterInfo('formio_action_plugin');
    $this->setCacheBackend($cache_backend, 'formio_action_plugins');
  }

  /**
   * ActionPlugin factory method.
   *
   * @param string $bin
   *   The cache bin for the plugin.
   */
  public static function create($bin = 'cache') {
    return new static(Module::getNamespaces(), _cache_get_object($bin));
  }

  /**
   * {@inheritdoc}
   */
  public static function action($id) {}

  /**
   * Makes a restful call to Form.io.
   *
   * @param string $token
   *   The x-token used for authentication.
   * @param string $uri
   *   The api uri.
   * @param string $data
   *   A json object containing the data submitted in the form.
   *
   * @return array
   *   The result of the rest call.
   */
  protected function rest_call($token, $uri, $data) {
    // Initialize the session.
    $curl = curl_init();

    // Setup options for the transfer.
    curl_setopt_array($curl, array(
      // Adding a query here for a 'Dry Run'.
      CURLOPT_URL => $uri, // . '?dryrun=1',
      CURLOPT_RETURNTRANSFER => TRUE,
      CURLOPT_ENCODING => "",
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 30,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => "POST",
      CURLOPT_POSTFIELDS => "$data",
      CURLOPT_HTTPHEADER => array(
        "cache-control: no-cache",
        "content-type: application/json",
        "x-token: " . $token,
      ),
    ));

    // Execute the session.
    $response = curl_exec($curl);

    // The last error, if any, for the current session.
    $error = curl_error($curl);

    // The session result containing the response and any errors.
    $result = array(
      'response' => $response,
      'error' => $error,
    );

    // Close the session.
    curl_close($curl);

    return $result;
  }

}
