<?php

namespace Drupal\formio\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\plug\Util\Module;

class FormioAction extends PluginBase implements ActionPluginInterface {

  /**
   * Used to define a menu item used by the form for a callback.
   *
   * @param array $items
   *   Menu items.
   */
  public function hook_menu(&$items) {
    $items['formio/%'] = array(
      'page callback' => $this->plugin['callback'],
      'page arguments' => array($this->plugin['name'], 1),
      'access callback' => $this->plugin['access'],
      'type' => MENU_CALLBACK,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function action($id) {

    // A json object containing the contents of the form submission.
    $data = file_get_contents('php://input');

    $request = formio_rest('form/' . $id);
    $results = $request->fetchKeyed('path');

    // We should always have a path but in case we don't throw an error.
    $path = $results->result['path'];
    if (!isset($path)) {
      // TODO: Cause an error for $result['error'] and see how it looks.
      drupal_json_output(json_encode('Example: Could not find a path'));
    }

    // The api uri for a submission.
    $uri = $results->project . '/'. $path . '/submission';

    // This is used to validate the request. This will be posted back to form.io
    // and the result will need to be a 200. Add a ?dryrun=1 query to the post.
    $token = getallheaders()['X-Token'];

    // Consider @see drupal_deliver_html_page().
    $result = $this->rest_call($token, $uri, $data);

    if (empty($result['error'])) {
      $output = $result['response'];
    }
    else {
      $output = $result['error'];
    }

    // Encode the output.
    drupal_json_output(json_encode($output));

    // Exit.
    drupal_exit();
  }

}
